/*
 *	Copyright 2012  Samsung Electronics Co., Ltd
 *
 *	Licensed under the Flora License, Version 1.0 (the "License");
 *	you may not use this file except in compliance with the License.
 *	You may obtain a copy of the License at
 *
 * 		http://floralicense.org/license/
 *
 *	Unless required by applicable law or agreed to in writing, software
 *	distributed under the License is distributed on an "AS IS" BASIS,
 *	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *	See the License for the specific language governing permissions and
 *	limitations under the License.
 */


#ifndef UG_MODULE_API
#define UG_MODULE_API __attribute__ ((visibility("default")))
#endif

#include "mgr-app-uigadget.h"
#include "mgr-app-common-debug.h"
#include "mgr-app-common-error.h"
#include "mgr-app-common-util.h"
#include "mgr-app-view-main.h"
#include "mgr-app-view-common.h"
#include "mgr-app-widget.h"

#define STR_VIEWTYPE				"viewtype"
#define STR_PKGNAME					"pkgname"
#define STR_MANAGE_APPLICATIONS		"manage-applications"
#define STR_APPLICATION_INFO		"application-info"

#define EDC_FILE					EDJDIR"/ug-setting-manage-applications-efl/ug-setting-manage-applications-efl.edj"

static void *on_create(ui_gadget_h ug, enum ug_mode mode, service_h service, void *priv)
{
	MGR_APP_BEGIN();

	struct ug_data *ugd = NULL;
	char *arg = NULL;
	char *arg2 = NULL;
	int ret = SERVICE_ERROR_NONE;

	retv_if(priv == NULL, NULL);

	bindtextdomain(PACKAGE, LOCALEDIR);

	ugd = priv;
	ugd->ug = ug;

	ugd->win_main = ug_get_parent_layout(ug);
	retv_if(ugd->win_main == NULL, NULL);

	ugd->bg = mgr_app_widget_create_bg(ugd->win_main);
	if (!ugd->bg) {
		MGR_APP_DEBUG_ERR("ugd->bg is null");
		MGR_APP_MEM_FREE(ugd->win_main);
		return NULL;
	}

	ugd->layout_main = mgr_app_widget_create_main_layout(ugd->win_main);
	if (!ugd->layout_main) {
		MGR_APP_DEBUG_ERR("ugd->layout_main is null");
		MGR_APP_MEM_FREE(ugd->win_main);
		MGR_APP_MEM_FREE(ugd->bg);
		MGR_APP_END();
		return NULL;
	}

	elm_object_part_content_set(ugd->layout_main, "elm.swallow.bg", ugd->bg);

	ugd->navi_bar = mgr_app_view_create_base_navigation(ugd->layout_main);
	if (!ugd->navi_bar) {
		MGR_APP_DEBUG_ERR("ugd->navi_bar is null");
		MGR_APP_MEM_FREE(ugd->win_main);
		MGR_APP_MEM_FREE(ugd->bg);
		MGR_APP_MEM_FREE(ugd->layout_main);
		MGR_APP_END();
		return NULL;
	}

	/* package manager listener */
	if (ugd->pc_listen) {
		pkgmgr_client_free(ugd->pc_listen);
		ugd->pc_listen = NULL;
	} else {
		ugd->pc_listen = pkgmgr_client_new(PC_LISTENING);
		if (!ugd->pc_listen) {
			MGR_APP_DEBUG_ERR("pc_listen is null");
			MGR_APP_MEM_FREE(ugd->win_main);
			MGR_APP_MEM_FREE(ugd->bg);
			MGR_APP_MEM_FREE(ugd->layout_main);
			MGR_APP_END();
			return NULL;
		}
	}

	ugd->uninstall = EINA_FALSE;

	ret = service_get_extra_data(service, STR_VIEWTYPE, &arg);
	switch (ret) {
	case SERVICE_ERROR_NONE:
		MGR_APP_DEBUG("SERVICE_ERROR_NONE. arg: %s", arg);
		break;
	case SERVICE_ERROR_INVALID_PARAMETER:
		MGR_APP_DEBUG_ERR("SERVICE_ERROR_INVALID_PARAMETER");
		break;
	case SERVICE_ERROR_KEY_NOT_FOUND:
		MGR_APP_DEBUG_ERR("SERVICE_ERROR_KEY_NOT_FOUND");
		break;
	case SERVICE_ERROR_OUT_OF_MEMORY:
		MGR_APP_DEBUG_ERR("SERVICE_ERROR_OUT_OF_MEMORY");
		break;
	case SERVICE_ERROR_INVALID_DATA_TYPE:
		MGR_APP_DEBUG_ERR("SERVICE_ERROR_INVALID_DATA_TYPE");
		break;
	default:
		MGR_APP_DEBUG_ERR("ret value has an error.");
		break;
	}

	elm_theme_extension_add(NULL, EDC_FILE);

	if (arg && !strncmp(arg, STR_MANAGE_APPLICATIONS, strlen(STR_MANAGE_APPLICATIONS))) {
		mgrapp_view_list_create((void*)ugd);
	} else if (arg && !strncmp(arg, STR_APPLICATION_INFO, strlen(STR_APPLICATION_INFO))) {
		ret = service_get_extra_data(service, STR_PKGNAME, &arg2);
		switch (ret) {
		case SERVICE_ERROR_NONE:
			MGR_APP_DEBUG("SERVICE_ERROR_NONE. arg: %s", arg2);
			break;
		case SERVICE_ERROR_INVALID_PARAMETER:
			MGR_APP_DEBUG_ERR("SERVICE_ERROR_INVALID_PARAMETER");
			break;
		case SERVICE_ERROR_KEY_NOT_FOUND:
			MGR_APP_DEBUG_ERR("SERVICE_ERROR_KEY_NOT_FOUND");
			break;
		case SERVICE_ERROR_OUT_OF_MEMORY:
			MGR_APP_DEBUG_ERR("SERVICE_ERROR_OUT_OF_MEMORY");
			break;
		case SERVICE_ERROR_INVALID_DATA_TYPE:
			MGR_APP_DEBUG_ERR("SERVICE_ERROR_INVALID_DATA_TYPE");
			break;
		default:
			MGR_APP_DEBUG_ERR("ret value has an error.");
			break;
		}	
		mgrapp_view_info_create(arg2, (void*)ugd);
	} else {
		MGR_APP_DEBUG_ERR("argument has an error.");
	}

	MGR_APP_END();
	return ugd->layout_main;
}

static void on_start(ui_gadget_h ug, service_h service, void *priv)
{
	MGR_APP_BEGIN();
	MGR_APP_END();
}

static void on_pause(ui_gadget_h ug, service_h service, void *priv)
{
	MGR_APP_BEGIN();
	
	mgr_app_doubletap_sensor_disable();

	MGR_APP_END();
}

static void on_resume(ui_gadget_h ug, service_h service, void *priv)
{
	MGR_APP_BEGIN();

	struct ug_data *ugd = NULL;
	view_type_t target_view = VIEW_TYPE_NONE;

	target_view = mgrapp_get_view_type();
	if (VIEW_TYPE_APP_INFO == target_view) {
		ret_if(priv == NULL);
		ugd = priv;
		mgrapp_view_info_update((void*)ugd);
	}
	
	mgr_app_doubletap_sensor_enable();

	MGR_APP_END();
}

static void on_destroy(ui_gadget_h ug, service_h service, void *priv)
{
	MGR_APP_BEGIN();

	ret_if(priv == NULL);

	struct ug_data *ugd = priv;

	mgr_app_doubletap_sensor_destroy();

	if (ugd->pc_listen) {
		pkgmgr_client_free(ugd->pc_listen);
		ugd->pc_listen = NULL;
	}

	if (ugd->layout_main) {
		evas_object_del(ugd->layout_main);
		ugd->layout_main = NULL;
	}

	MGR_APP_END();
}

static void on_message(ui_gadget_h ug, service_h msg, service_h service, void *priv)
{
	MGR_APP_BEGIN();
	MGR_APP_END();
}

static void on_event(ui_gadget_h ug, enum ug_event event, service_h service, void *priv)
{
	MGR_APP_BEGIN();
	switch (event) {
	case UG_EVENT_LOW_MEMORY:
		break;
	case UG_EVENT_LOW_BATTERY:
		break;
	case UG_EVENT_LANG_CHANGE:
		break;
	case UG_EVENT_ROTATE_PORTRAIT:
		break;
	case UG_EVENT_ROTATE_PORTRAIT_UPSIDEDOWN:
		break;
	case UG_EVENT_ROTATE_LANDSCAPE:
		break;
	case UG_EVENT_ROTATE_LANDSCAPE_UPSIDEDOWN:
		break;
	default:
		break;
	}
	MGR_APP_END();
}

static void on_key_event(ui_gadget_h ug, enum ug_key_event event, service_h service, void *priv)
{
	MGR_APP_BEGIN();

	switch (event) {
	case UG_KEY_EVENT_END:
		ug_destroy_me(ug);
		break;
	default:
		break;
	}
	MGR_APP_END();
}

UG_MODULE_API int UG_MODULE_INIT(struct ug_module_ops *ops)
{
	MGR_APP_BEGIN();
	struct ug_data *ugd = NULL;

	retv_if(ops == NULL, -1);

	MGR_APP_MEM_MALLOC(ugd, (1), struct ug_data);

	ops->create 	= on_create;
	ops->start 		= on_start;
	ops->pause 		= on_pause;
	ops->resume 	= on_resume;
	ops->destroy 	= on_destroy;
	ops->message 	= on_message;
	ops->event	 	= on_event;
	ops->key_event 	= on_key_event;
	ops->priv 		= ugd;
	ops->opt 		= UG_OPT_INDICATOR_ENABLE;

	MGR_APP_END();
	return 0;
}

UG_MODULE_API void UG_MODULE_EXIT(struct ug_module_ops *ops)
{
	MGR_APP_BEGIN();
	struct ug_data *ugd = NULL;

	ret_if(ops == NULL);

	ugd = ops->priv;
	MGR_APP_MEM_FREE(ugd);

	MGR_APP_END();
}

UG_MODULE_API int setting_plugin_reset(service_h service, void *priv)
{
	MGR_APP_BEGIN();
	/* nothing to do for Setting>Reset */
	MGR_APP_END();
	return 0;
}
